@@ -115,7 +115,7 @@ error:
 /* Group methods */
 
 inline ParticleRefObject *
-ParticleRefObject_New(GroupObject *pgroup, Particle *p);
+ParticleRefObject_New(PyObject *parent, Particle *p);
 
 /* Create a new particle in the group from a template */
 static ParticleRefObject *
@@ -144,7 +144,7 @@ ParticleGroup_new(GroupObject *self, PyObject *ptemplate)
 	if (!success)
 		return NULL;
 
-	return ParticleRefObject_New(self, pnew);
+	return ParticleRefObject_New((PyObject *)self, pnew);
 }
 
 static inline int
@@ -200,7 +200,7 @@ ParticleGroup_iter(PyObject *self)
 		PyErr_NoMemory();
 		return NULL;
 	}
-	piter->pgroup = group;
+	piter->parent = (PyObject *)group;
 	Py_INCREF(group);
 	piter->p = group->plist->p;
 	piter->iteration = group->iteration;
@@ -480,7 +480,7 @@ static PyTypeObject ParticleGroup_Type = {
 	0,			/*tp_hash*/
 	0,                      /*tp_call*/
 	0,                      /*tp_str*/
-	PyObject_GenericGetAttr, /*tp_getattro*/
+	0,                      /*tp_getattro*/
 	0,                      /*tp_setattro*/
 	0,                      /*tp_as_buffer*/
 	Py_TPFLAGS_DEFAULT,     /*tp_flags*/
@@ -510,17 +510,33 @@ static PyTypeObject ParticleGroup_Type = {
 
 static PyTypeObject Vector_Type;
 
+static VectorObject *vector_pool = NULL;
+static int vector_pool_count = 0;
+#define MAX_VECTOR_POOL 50
+
+#define ParticleRef_INVALID(v) \
+	((v)->parent != NULL && GroupObject_CHECK((v)->parent) \
+	 && (v)->iteration != ((GroupObject *)((v)->parent))->iteration)
+
 static void
 Vector_dealloc(VectorObject *self)
 {
-	if (self->pgroup != NULL)
-		Py_CLEAR(self->pgroup);
-	PyObject_Del(self);
+	if (self->parent != NULL)
+		Py_CLEAR(self->parent);
+	if (vector_pool_count < MAX_VECTOR_POOL) {
+		self->parent = (PyObject *)vector_pool;
+		vector_pool = self;
+		vector_pool_count++;
+	} else {
+		PyObject_Del(self);
+	}
 }
 
-/* Create a new vector object for the group and vector struct specified */
-static VectorObject *
-Vector_new(GroupObject *pgroup, Vec3 *vec, int length)
+/* Create a new vector object for the parent object and vector struct specified 
+ * The parent object may be NULL if there is none
+ */
+VectorObject *
+Vector_new(PyObject *parent, Vec3 *vec, int length)
 {
 	VectorObject *newvec;
 
@@ -528,16 +544,26 @@ Vector_new(GroupObject *pgroup, Vec3 *vec, int length)
 		PyErr_SetString(PyExc_ValueError, "expected length 3 or 4");
 		return NULL;
 	}
-
-	newvec = PyObject_New(VectorObject, &Vector_Type);
-	if (newvec == NULL) {
-		PyErr_NoMemory();
-		return NULL;
+	if (vector_pool_count) {
+		newvec = vector_pool;
+		Py_INCREF(newvec);
+		vector_pool = (VectorObject *)vector_pool->parent;
+		vector_pool_count--;
+	} else {
+		newvec = PyObject_New(VectorObject, &Vector_Type);
+		if (newvec == NULL) {
+			PyErr_NoMemory();
+			return NULL;
+		}
 	}
-	newvec->pgroup = pgroup;
-	if (pgroup != NULL) {
-		Py_INCREF(pgroup);
-		newvec->iteration = pgroup->iteration;
+	newvec->parent = parent;
+	if (parent != NULL) {
+		Py_INCREF(parent);
+		if (GroupObject_CHECK(parent)) {
+			newvec->iteration = ((GroupObject *)parent)->iteration;
+		} else {
+			newvec->iteration = 0;
+		}
 	} else {
 		newvec->iteration = 0;
 	}
@@ -546,44 +572,18 @@ Vector_new(GroupObject *pgroup, Vec3 *vec, int length)
 	return newvec;
 }
 
-static PyObject *
-Vector_getattr(VectorObject *self, char *name)
-{
-	if (self->pgroup != NULL && self->iteration != self->pgroup->iteration) {
-		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
-		return NULL;
-	}
-	
-	if (strlen(name) != 1) {
-		PyErr_SetString(PyExc_AttributeError, name);
-		return NULL;
-	}
-
-	switch (name[0]) {
-		case 'r': case 'x': return PyFloat_FromDouble(self->vec->x);
-		case 'g': case 'y': return PyFloat_FromDouble(self->vec->y);
-		case 'b': case 'z': return PyFloat_FromDouble(self->vec->z);
-		case 'a': return PyFloat_FromDouble(self->color->a);
-	}
-
-	PyErr_SetString(PyExc_AttributeError, name);
-	return NULL;
-}
-
 static int
 Vector_setattr(VectorObject *self, char *name, PyObject *v)
 {
 	int result;
-	if (self->pgroup != NULL && self->iteration != self->pgroup->iteration) {
+	if (ParticleRef_INVALID(self)) {
 		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
 		return -1;
 	}
-	
 	if (strlen(name) != 1) {
 		PyErr_SetString(PyExc_AttributeError, name);
 		return -1;
 	}
-
 	v = PyNumber_Float(v);
 	if (v == NULL)
 		return -1;
@@ -614,20 +614,19 @@ Vector_setattr(VectorObject *self, char *name, PyObject *v)
 static PyObject *
 Vector_repr(VectorObject *self)
 {
-	const int bufsize = 255;
-	char buf[bufsize];
-	if (self->pgroup == NULL || self->iteration == self->pgroup->iteration) {
+	char buf[255];
+	if (!ParticleRef_INVALID(self)) {
 		buf[0] = 0; /* paranoid */
 		if (self->length == 3) 
-			snprintf(buf, bufsize, "Vector(%.1f, %.1f, %.1f)",
+			PyOS_snprintf(buf, 255, "Vector(%.1f, %.1f, %.1f)",
 				self->vec->x, self->vec->y, self->vec->z);
 		else
-			snprintf(buf, bufsize, "Color(%.1f, %.1f, %.1f, %.1f)",
+			PyOS_snprintf(buf, 255, "Color(%.1f, %.1f, %.1f, %.1f)",
 				self->color->r, self->color->g, self->color->b, self->color->a);
 		return PyString_FromString(buf);
 	} else {
 		return PyString_FromFormat(
-			"<INVALID vector of group %p>", self->pgroup);
+			"<INVALID vector of group %p>", self->parent);
 	}
 }
 
@@ -640,7 +639,7 @@ Vector_length(VectorObject *self)
 static PyObject *
 Vector_getitem(VectorObject *self, Py_ssize_t index)
 {
-	if (self->pgroup != NULL && self->iteration != self->pgroup->iteration) {
+	if (ParticleRef_INVALID(self)) {
 		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
 		return NULL;
 	}
@@ -658,6 +657,11 @@ Vector_getitem(VectorObject *self, Py_ssize_t index)
 static int
 Vector_assitem(VectorObject *self, Py_ssize_t index, PyObject *v)
 {
+	if (ParticleRef_INVALID(self)) {
+		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
+		return -1;
+	}
+
 	switch (index) {
 		case 0: return Vector_setattr(self, "x", v);
 		case 1: return Vector_setattr(self, "y", v);
@@ -668,6 +672,46 @@ Vector_assitem(VectorObject *self, Py_ssize_t index, PyObject *v)
 	return -1;
 }
 
+static PyObject *
+Vector_clamp(VectorObject *self, PyObject *args)
+{
+	float min, max;
+
+	if (ParticleRef_INVALID(self)) {
+		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
+		return NULL;
+	}
+
+	if (!PyArg_ParseTuple(args, "ff:clamp",  &min, &max))
+		return NULL;
+
+	if (min > max) {
+		PyErr_Format(PyExc_ValueError, "clamp: Expected min <= max");
+		return NULL;
+	}
+
+	if (self->length == 3) {
+		self->vec->x = (self->vec->x < min) ? min : self->vec->x;
+		self->vec->x = (self->vec->x > max) ? max : self->vec->x;
+		self->vec->y = (self->vec->y < min) ? min : self->vec->y;
+		self->vec->y = (self->vec->y > max) ? max : self->vec->y;
+		self->vec->z = (self->vec->z < min) ? min : self->vec->z;
+		self->vec->z = (self->vec->z > max) ? max : self->vec->z;
+	} else {
+		self->color->r = (self->color->r < min) ? min : self->color->r;
+		self->color->r = (self->color->r > max) ? max : self->color->r;
+		self->color->g = (self->color->g < min) ? min : self->color->g;
+		self->color->g = (self->color->g > max) ? max : self->color->g;
+		self->color->b = (self->color->b < min) ? min : self->color->b;
+		self->color->b = (self->color->b > max) ? max : self->color->b;
+		self->color->a = (self->color->a < min) ? min : self->color->a;
+		self->color->a = (self->color->a > max) ? max : self->color->a;
+	}
+
+	Py_INCREF(self);
+	return (PyObject *)self;
+}
+
 static PySequenceMethods Vector_as_sequence = {
 	(lenfunc)Vector_length,	/* sq_length */
 	0,		/*sq_concat*/
@@ -677,6 +721,36 @@ static PySequenceMethods Vector_as_sequence = {
 	(ssizeobjargproc)Vector_assitem,	/* sq_ass_item */
 };
 
+static PyMethodDef Vector_methods[] = {
+	{"clamp", (PyCFunction)Vector_clamp, METH_VARARGS,
+		PyDoc_STR("clamp(min, max) -> vector\n"
+			"Clamp all values of the vector between min and max\n"
+			"return the resulting vector")},
+	{NULL, NULL}
+};
+
+static PyObject *
+Vector_getattr(VectorObject *self, PyObject *o)
+{
+	char *name = PyString_AS_STRING(o);
+
+	if (ParticleRef_INVALID(self)) {
+		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
+		return NULL;
+	}
+	
+	if (strlen(name) == 1) {
+		switch (name[0]) {
+			case 'r': case 'x': return PyFloat_FromDouble(self->vec->x);
+			case 'g': case 'y': return PyFloat_FromDouble(self->vec->y);
+			case 'b': case 'z': return PyFloat_FromDouble(self->vec->z);
+			case 'a': return PyFloat_FromDouble(self->color->a);
+		}
+	}
+
+	return Py_FindMethod(Vector_methods, (PyObject *)self, name);
+}
+
 PyDoc_STRVAR(Vector__doc__, "Vector swizzler");
 
 static PyTypeObject Vector_Type = {
@@ -690,7 +764,7 @@ static PyTypeObject Vector_Type = {
 	/* methods */
 	(destructor)Vector_dealloc, /*tp_dealloc*/
 	0,			            /*tp_print*/
-	(getattrfunc)Vector_getattr, /*tp_getattr*/
+	0,                      /*tp_getattr*/
 	(setattrfunc)Vector_setattr, /*tp_setattr*/
 	0,			            /*tp_compare*/
 	(reprfunc)Vector_repr,  /*tp_repr*/
@@ -700,7 +774,7 @@ static PyTypeObject Vector_Type = {
 	0,			            /*tp_hash*/
 	0,                      /*tp_call*/
 	0,                      /*tp_str*/
-	0,                      /*tp_getattro*/
+	(getattrofunc)Vector_getattr, /*tp_getattro*/
 	0,                      /*tp_setattro*/
 	0,                      /*tp_as_buffer*/
 	Py_TPFLAGS_DEFAULT,     /*tp_flags*/
@@ -711,7 +785,7 @@ static PyTypeObject Vector_Type = {
 	0,                      /*tp_weaklistoffset*/
 	0,                      /*tp_iter*/
 	0,                      /*tp_iternext*/
-	0,                      /*tp_methods*/
+	Vector_methods,         /*tp_methods*/
 	0,                      /*tp_members*/
 	0,                      /*tp_getset*/
 	0,                      /*tp_base*/
@@ -728,31 +802,51 @@ static PyTypeObject Vector_Type = {
 
 /* --------------------------------------------------------------------- */
 
+static ParticleRefObject *pproxy_pool = NULL;
+static int pproxy_pool_count = 0;
+#define MAX_PPROXY_POOL 50
 
 static void
 ParticleProxy_dealloc(ParticleRefObject *self)
 {
-	if (self->pgroup != NULL)
-		Py_CLEAR(self->pgroup);
-	PyObject_Del(self);
+	if (self->parent != NULL)
+		Py_CLEAR(self->parent);
+	if (pproxy_pool_count < MAX_PPROXY_POOL) {
+		self->parent = (PyObject *)pproxy_pool;
+		pproxy_pool = self;
+		pproxy_pool_count++;
+	} else {
+		PyObject_Del(self);
+	}
 }
 
 /* Create a new particle reference object for the given group and index
  * no range checking is done
  */
 inline ParticleRefObject *
-ParticleRefObject_New(GroupObject *pgroup, Particle *p)
+ParticleRefObject_New(PyObject *parent, Particle *p)
 {
 	ParticleRefObject *pproxy;
-	pproxy = PyObject_New(ParticleRefObject, &ParticleProxy_Type);
-	if (pproxy == NULL) {
-		PyErr_NoMemory();
-		return NULL;
+	if (pproxy_pool_count) {
+		pproxy = pproxy_pool;
+		Py_INCREF(pproxy);
+		pproxy_pool = (ParticleRefObject *)pproxy->parent;
+		pproxy_pool_count--;
+	} else {
+		pproxy = PyObject_New(ParticleRefObject, &ParticleProxy_Type);
+		if (pproxy == NULL) {
+			PyErr_NoMemory();
+			return NULL;
+		}
 	}
-	pproxy->pgroup = pgroup;
-	if (pgroup != NULL) {
-		Py_INCREF(pgroup);
-		pproxy->iteration = pgroup->iteration;
+	pproxy->parent = parent;
+	if (parent != NULL) {
+		Py_INCREF(parent);
+		if (GroupObject_CHECK(parent)) {
+			pproxy->iteration = ((GroupObject *)parent)->iteration;
+		} else {
+			pproxy->iteration = 0;
+		}
 	} else {
 		pproxy->iteration = 0;
 	}
@@ -762,7 +856,7 @@ ParticleRefObject_New(GroupObject *pgroup, Particle *p)
 
 static inline int
 ParticleRefObject_IsValid(ParticleRefObject *pref) {
-	if (pref->pgroup == NULL || pref->iteration == pref->pgroup->iteration) {
+	if (!ParticleRef_INVALID(pref)) {
 		return 1;
 	} else {
 		PyErr_SetString(InvalidParticleRefError, "Invalid particle reference");
@@ -792,15 +886,16 @@ ParticleProxy_getattr(ParticleRefObject *self, char *name)
 		PyErr_SetString(PyExc_AttributeError, name);
 		return NULL;
 	}
+
 	switch (attr_no) {
-		case 0: return (PyObject *)Vector_new(self->pgroup, &self->p->position, 3);
-		case 1: return (PyObject *)Vector_new(self->pgroup, &self->p->velocity, 3);
-		case 2: return (PyObject *)Vector_new(self->pgroup, &self->p->size, 3);
-		case 3: return (PyObject *)Vector_new(self->pgroup, &self->p->up, 3);
-		case 4: return (PyObject *)Vector_new(self->pgroup, &self->p->rotation, 3);
-		case 5: return (PyObject *)Vector_new(self->pgroup, &self->p->last_position, 3);
-		case 6: return (PyObject *)Vector_new(self->pgroup, &self->p->last_velocity, 3);
-		case 7: return (PyObject *)Vector_new(self->pgroup, (Vec3 *)&self->p->color, 4);
+		case 0: return (PyObject *)Vector_new(self->parent, &self->p->position, 3);
+		case 1: return (PyObject *)Vector_new(self->parent, &self->p->velocity, 3);
+		case 2: return (PyObject *)Vector_new(self->parent, &self->p->size, 3);
+		case 3: return (PyObject *)Vector_new(self->parent, &self->p->up, 3);
+		case 4: return (PyObject *)Vector_new(self->parent, &self->p->rotation, 3);
+		case 5: return (PyObject *)Vector_new(self->parent, &self->p->last_position, 3);
+		case 6: return (PyObject *)Vector_new(self->parent, &self->p->last_velocity, 3);
+		case 7: return (PyObject *)Vector_new(self->parent, (Vec3 *)&self->p->color, 4);
 		case 8: return Py_BuildValue("f", self->p->mass);
 		case 9: return Py_BuildValue("f", self->p->age);
 	};
@@ -893,18 +988,17 @@ ParticleProxy_setattr(ParticleRefObject *self, char *name, PyObject *v)
 static PyObject *
 ParticleProxy_repr(ParticleRefObject *self)
 {
-	const int bufsize = 1024;
-	char buf[bufsize];
+	char buf[1024];
 
 	if (ParticleRefObject_IsValid(self)) {
 		buf[0] = 0; /* paranoid */
-		snprintf(buf, bufsize, "<Particle %lu of group 0x%lx: "
+		PyOS_snprintf(buf, 1024, "<Particle %lu of group 0x%lx: "
 			"position=(%.1f, %.1f, %.1f) velocity=(%.1f, %.1f, %.1f) "
 			"color=(%.1f, %.1f, %.1f, %.1f) size=(%.1f, %.1f, %.1f) "
 			"up=(%.1f, %.1f, %.1f) rotation=(%.1f, %.1f, %.1f) "
 			"last_position=(%.1f, %.1f, %.1f) last_velocity=(%.1f, %.1f, %.1f) "
 			"mass=%.1f age=%.1f>",
-			(unsigned long)(self->p), (unsigned long)self->pgroup,
+			(unsigned long)(self->p), (unsigned long)self->parent,
 			self->p->position.x, self->p->position.y, self->p->position.z,
 			self->p->velocity.x, self->p->velocity.y, self->p->velocity.z,
 			self->p->color.r, self->p->color.g, self->p->color.b, self->p->color.a,
@@ -917,7 +1011,7 @@ ParticleProxy_repr(ParticleRefObject *self)
 		return PyString_FromString(buf);
 	} else {
 		return PyString_FromFormat("<INVALID Particle %lu of group %p>",
-			(unsigned long)(self->p), self->pgroup);
+			(unsigned long)(self->p), self->parent);
 	}
 }
 
@@ -977,7 +1071,7 @@ static PyTypeObject ParticleProxy_Type = {
 static void
 ParticleIter_dealloc(ParticleRefObject *self)
 {
-	Py_CLEAR(self->pgroup);
+	Py_CLEAR(self->parent);
 	PyObject_Del(self);
 }
 
@@ -985,11 +1079,13 @@ static PyObject *
 ParticleIter_next(ParticleRefObject *self)
 {
 	Particle *lastp;
+	GroupObject *pgroup;
 
 	if (!ParticleRefObject_IsValid(self))
 		return NULL;
 
-	lastp = &self->pgroup->plist->p[GroupObject_ActiveCount(self->pgroup)];
+	pgroup = (GroupObject *)self->parent;
+	lastp = &pgroup->plist->p[GroupObject_ActiveCount(pgroup)];
 
 	/* Scan to the next active particle */
 	while (self->p < lastp && !Particle_IsAlive(*self->p)) {
@@ -997,7 +1093,7 @@ ParticleIter_next(ParticleRefObject *self)
 	}
 	
 	if (self->p < lastp) {
-		return (PyObject *)ParticleRefObject_New(self->pgroup, self->p++);
+		return (PyObject *)ParticleRefObject_New(self->parent, self->p++);
 	} else {
 		/* End of iteration */
 		return NULL;
@@ -1027,7 +1123,7 @@ static PyTypeObject ParticleIter_Type = {
 	0,			            /*tp_hash*/
 	0,                      /*tp_call*/
 	0,                      /*tp_str*/
-	PyObject_GenericGetAttr, /*tp_getattro*/
+	0,                      /*tp_getattro*/
 	0,                      /*tp_setattro*/
 	0,                      /*tp_as_buffer*/
 	Py_TPFLAGS_DEFAULT,     /*tp_flags*/
@@ -1036,7 +1132,7 @@ static PyTypeObject ParticleIter_Type = {
 	0,                      /*tp_clear*/
 	0,                      /*tp_richcompare*/
 	0,                      /*tp_weaklistoffset*/
-	PyObject_SelfIter,       /*tp_iter*/
+	0,                      /*tp_iter*/
 	(iternextfunc)ParticleIter_next,  /*tp_iternext*/
 	0,                      /*tp_methods*/
 	0,                      /*tp_members*/
@@ -1060,9 +1156,10 @@ initgroup(void)
 {
 	PyObject *m;
 
-	/* Bind tp_new and tp_alloc here to appease certain compilers */
+	/* Bind external consts here to appease certain compilers */
 	ParticleGroup_Type.tp_alloc = PyType_GenericAlloc;
 	ParticleGroup_Type.tp_new = PyType_GenericNew;
+	ParticleGroup_Type.tp_getattro = PyObject_GenericGetAttr;
 	if (PyType_Ready(&ParticleGroup_Type) < 0)
 		return;
 
@@ -1071,6 +1168,8 @@ initgroup(void)
 		return;
 
 	ParticleIter_Type.tp_alloc = PyType_GenericAlloc;
+	ParticleIter_Type.tp_getattro = PyObject_GenericGetAttr;
+	ParticleIter_Type.tp_iter = PyObject_SelfIter;
 	if (PyType_Ready(&ParticleIter_Type) < 0)
 		return;
 
